/* -- $Id: Driver.java,v 1.7 2003/11/06 07:03:46 kshaikh Exp $ --
 Copyright (C) 2001 Auriga Logic <http://www.aurigalogic.com>
 Author: Khurshidali Shaikh <kshaikh@aurigalogic.com>

 ============================================================================
		    GNU GENERAL PUBLIC LICENSE Version 2, June 1991
 ============================================================================

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
package com.aurigalogic.aurigadoc.core;

import java.util.Properties;

import com.aurigalogic.aurigadoc.logger.*;

/**
 * This class is the main driver class that takes care of conversion to 
 * various output format. Use this class for doing the conversion.<br />
 * e.g.<br />
 * Driver driver = new Driver(); // intantiate a driver<br />
 * driver.setFormat(Driver.FORMAT_HTML); // set the output format <br />
 * driver.setInputFile(String inputFile); // set the input file path<br />
 * driver.setOutputFile(String outputFile); // set teh output file/dir 
 * path<br />
 * driver.setLogger(Logger logger); //optional<br />
 * driver.run(); // run the driver
 *
 * @author <a href="mailto:kshaikh@aurigalogic.com">Khurshidali Shaikh</a>
 * @version $Revision: 1.7 $ $Date: 2003/11/06 07:03:46 $
 */

public class Driver {

	/** Constant for the XML output format */
	public static final int FORMAT_HTML = 1;

	/** Constant for the MULTI-HTML output format */
	public static final int FORMAT_MHTML = 2;

	/** Constant for the DHTML output format */
	public static final int FORMAT_DHTML = 3;

	/** Constant for the PDF output format */
	public static final int FORMAT_PDF = 4;

	/** Constant for the PS output format */
	public static final int FORMAT_PS = 5;

	/** Constant for the RTF output format */
	public static final int FORMAT_RTF = 6;

	/** Constant for the AWT output format */
	public static final int FORMAT_AWT = 7;

	/** Constant for the FO output format */
	public static final int FORMAT_FO = 8;

	/** Constant for the JHELP output format */
	public static final int FORMAT_JHELP = 9;

	/** Constant for the CHM output format */
	public static final int FORMAT_CHM = 11;

	/** Constant for the man source output format */
	public static final int FORMAT_MAN = 12;

	/** Constant for the mht output format */
	public static final int FORMAT_MHT = 13;

	/** Constant for the Oracle Help For Java (OHJ) output format */
	public static final int FORMAT_OHJ = 14;

	/** Output fromat required */
	private int format;

	/** The input file path */
	private String inputFile;

	/** The output file/dir path */
	private String outputFile;

	/** The logger */
	private Logger logger;

	/** Additional parameters used by a specific converter */
	private Properties parameters = new Properties();

	/**
	 * Default constructor.
	 */
	public Driver() {
	}

	/**
	 * Set the output format.
	 *
	 * @param format the output format.
	 */
	public void setFormat(int format) {
		this.format = format;
	}

	/**
	 * Set the output file.
	 *
	 * @param outFile the output file path.
	 */
	public void setOutputFile(String outFile) {
		this.outputFile = outFile;
	}

	/**
	 * Set the input file.
	 *
	 * @param inFile the input file path.
	 */
	public void setInputFile(String inFile) {
		this.inputFile = inFile;
	}

	/**
	 * Set the logger.
	 *
	 * @param logger the logger to use for logging.
	 */
	public void setLogger(Logger logger) {
		this.logger = logger;
	}

	/**
	 * Set additional parameters.
	 *
	 * @param parameters additional parameters for the specific
	 * converter.
	 */
	public void setParameters(Properties parameters) {
		this.parameters = parameters;
	}


	/**
	 * Run the converter.
	 * @throws Exception in case of an error.
	 */
	public void run() throws Exception {
		ADConverter converter = getConverter();
		converter.setInputFile(inputFile);
		converter.setLogger(logger);
		if (format !=  FORMAT_AWT) {
			converter.setOutputFile(outputFile);
		}
		converter.setParameters(parameters);
		converter.convert();
	}

	/**
	 * Get the converter instance for the specified output format.
	 */
	private ADConverter getConverter() throws Exception {

		ADConverter converter = null;
		String converterClass = "com.aurigalogic.aurigadoc.core.";
		switch (format) {
			case FORMAT_HTML:
				converterClass += "HTMLConverter";	
				break;
			case FORMAT_MHTML:
				converterClass += "MHTMLConverter";	
				break;
			case FORMAT_DHTML:
				converterClass += "DHTMLConverter";	
				break;
			case FORMAT_JHELP:
				converterClass += "JHelpConverter";	
				break;
			case FORMAT_PDF:
				converterClass += "PDFConverter";
				break;
			case FORMAT_PS:
				converterClass += "PSConverter";
				break;
			case FORMAT_RTF:
				converterClass += "RTFConverter";
				break;
			case FORMAT_AWT:
				converterClass += "AWTConverter";
				break;
			case FORMAT_FO:
				converterClass += "FOConverter";
				break;
			case FORMAT_CHM:
				converterClass += "CHMConverter";
				break;
			case FORMAT_MAN:
				converterClass += "MANConverter";
				break;
			case FORMAT_MHT:
				converterClass += "MHTConverter";
				break;
			case FORMAT_OHJ:
				converterClass += "OHJConverter";
				break;
			default:
				throw new IllegalArgumentException("Unknown output format.");
		}
		converter = 
			(ADConverter) Class.forName(converterClass).newInstance();
		return converter;
	}


}
