/* -- $Id: JHelpConverter.java,v 1.10 2003/10/16 15:25:10 kshaikh Exp $ --
 Copyright (C) 2001 Auriga Logic <http://www.aurigalogic.com>
 Author: Khurshidali Shaikh <kshaikh@aurigalogic.com>

 ============================================================================
		    GNU GENERAL PUBLIC LICENSE Version 2, June 1991
 ============================================================================

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
package com.aurigalogic.aurigadoc.core;

import java.io.*;

import com.aurigalogic.aurigadoc.util.JHelpViewer;
import com.aurigalogic.aurigadoc.util.ADFileFilter;

/**
 * This class is used for converting AurigaDoc document to java help 
 * files(helpset file, map file, toc file, search db).
 * 
 * @author <a href="mailto:kshaikh@aurigalogic.com">Khurshidali Shaikh</a>
 * @version $Revision: 1.10 $ $Date: 2003/10/16 15:25:10 $
 */

public class JHelpConverter extends MHTMLConverter {

	/** Identifier for specifying the xsl which generates the helpset file */
	public static final String HELPSET_XSL = "helpset_xsl";

	/** Identifier for specifying the xsl which generates the toc file */
	public static final String TOC_XSL = "toc_xsl";

	/** Identifier for specifying the xsl which generates the map file */
	public static final String MAP_XSL = "map_xsl";

	/** Identifier for setting the indexing option */
	public static final String INDEX_FILES = "index_files";

	/** Identifier for setting the viewing option */
	public static final String LAUNCH_VIEWER = "launch_viewer";

	private boolean indexFiles = false;

	private boolean launchViewer = false;

	private static final String JHELP_INDEXER_CONFIG_FILE = "jhelpconfig.cfg";

	/**
	 * Main method which does the conversion.
	 * @throws Exception in case and error occurs
	 */
	public void convert() throws Exception {
		readOptions();
		logInfo("Converting to JAVA HELP..");
		
		// generate the multiple html files
		convertToMHTML(outputFile);
			
		// make images dir in outputFile to copy images
		File imageDir = new File(outputFile + "/images");
		if (!imageDir.exists()) {
			logInfo("Creating dir " + imageDir.getName() 
				+ " for viewer images.. ");
			imageDir.mkdir();
		}

		// copy images for the java help viewer to this folder
		logInfo("Copying images to images dir..");
		copyFiles(AURIGADOC_HOME + "/images", outputFile + "/images");
		
		// copy icons for the java help viewer to this folder
		logInfo("Copying icons to icons dir..");
		copyFiles(AURIGADOC_HOME + "/icons", outputFile + "/icons");

		// generate the hs file
		FileOutputStream fos = new FileOutputStream(outputFile + "/jhelp.hs");
		logInfo("Generating java help help set file ..");
		doTransform(new FileInputStream(inputFile), 
			fos, getXSLForKey(HELPSET_XSL, XML2JHELPHS), OUTPUT_XML);
		fos.close();
			
		// generate the toc file
		fos = new FileOutputStream(outputFile + "/jhelptoc.xml");
		logInfo("Generating java help toc file..");
		doTransform(new FileInputStream(inputFile), 
			fos, getXSLForKey(TOC_XSL, XML2JHELPTOC), OUTPUT_XML);
		fos.close();

		// generate the map file
		fos = new FileOutputStream(outputFile + "/Map.jhm");
		logInfo("Generating java help map file..");
		doTransform(new FileInputStream(inputFile), 
			fos, getXSLForKey(MAP_XSL, XML2JHELPMAP), OUTPUT_XML);
		fos.close();

		if (indexFiles) {
			logInfo("Indexing files for use in java help ..");
			doIndexing(outputFile);
		}

		if (launchViewer) {
			logInfo("Launching files in a java help viewer..");
			launchHelpViewer(outputFile);
		}
	}


	private void doIndexing(String outDir) throws Exception {
		// index files
		if (!outDir.endsWith(File.separator)) {
			outDir += File.separator;
		}

		// create the config file.
		String indexerConfigFilePath = outDir + JHELP_INDEXER_CONFIG_FILE;
		createJHelpConfigFile(outDir, indexerConfigFilePath);

		File htmlDir = new File(outDir);
		ADFileFilter htmlFileFilter = new ADFileFilter(".html");
		File[] files = htmlDir.listFiles(htmlFileFilter);
		String[] indexerOptions = new String[4 + files.length];
		indexerOptions[0] = "-c";
		indexerOptions[1] = indexerConfigFilePath;
		indexerOptions[2] = "-db";
		indexerOptions[3] = new File(outDir 
			+ "/JavaHelpSearch").getAbsolutePath();

		for (int i = 0; i < files.length; i++) {
			indexerOptions[4 + i] = files[i].getAbsolutePath();
		}
		com.sun.java.help.search.Indexer.main(indexerOptions);

		// delete the config file.
		deleteJHelpConfigFile(indexerConfigFilePath);
	}

	/**
	 * Creates the jhelp file config file.
	 */
	private void createJHelpConfigFile(String outDir, 
			String indexerConfigFilePath) throws Exception {
		String pathToRemove = new File(outDir).getAbsolutePath();
		if (!pathToRemove.endsWith(File.separator)) {
			pathToRemove += File.separator;
		}
		FileWriter fw = new FileWriter(indexerConfigFilePath);
		PrintWriter pw = new PrintWriter(fw, true);
		pw.println("IndexRemove " + pathToRemove);
		pw.close();
		fw.close();
	}

	/**
	 * Deletes the jhelp file config file.
	 */
	private void deleteJHelpConfigFile(String indexerConfigFilePath) {
		try {
			File file = new File(indexerConfigFilePath);
			file.delete();
		} catch (Exception e) {
			// do nothing
		}
	}

	private void launchHelpViewer(String outDir) {
		new JHelpViewer(outDir, "jhelp");
	}

	private void readOptions() {
		String indexFilesFlag = getParameter(INDEX_FILES);
		if (indexFilesFlag != null && indexFilesFlag.equals("y")) {
			indexFiles = true;
		}

		String launchViewerFlag = getParameter(LAUNCH_VIEWER);
		if (launchViewerFlag != null && launchViewerFlag.equals("y")) {
			launchViewer = true;
		}
	}
}
