/* -- $Id: OHJConverter.java,v 1.2 2003/11/06 10:06:16 kshaikh Exp $ --
 Copyright (C) 2001 Auriga Logic <http://www.aurigalogic.com>
 Author: Khurshidali Shaikh <kshaikh@aurigalogic.com>

 ============================================================================
		    GNU GENERAL PUBLIC LICENSE Version 2, June 1991
 ============================================================================

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
package com.aurigalogic.aurigadoc.core;

import java.io.*;
import java.lang.reflect.*;

import com.aurigalogic.aurigadoc.util.JHelpViewer;
import com.aurigalogic.aurigadoc.util.ADFileFilter;

/**
 * This class is used for converting AurigaDoc document to Oracle Help 
 * for java (OHJ) format(helpset file, map file, toc file, search db).
 * 
 * @author <a href="mailto:kshaikh@aurigalogic.com">Khurshidali Shaikh</a>
 * @version $Revision: 1.2 $ $Date: 2003/11/06 10:06:16 $
 */

public class OHJConverter extends MHTMLConverter {

	/** Identifier for specifying the xsl which generates the helpset file */
	public static final String HELPSET_XSL = "helpset_xsl";

	/** Identifier for specifying the xsl which generates the toc file */
	public static final String TOC_XSL = "toc_xsl";

	/** Identifier for specifying the xsl which generates the map file */
	public static final String MAP_XSL = "map_xsl";

	/** Identifier for setting the indexing option */
	public static final String INDEX_FILES = "index_files";

	/** Identifier for setting the viewing option */
	public static final String LAUNCH_VIEWER = "launch_viewer";

	private boolean indexFiles = false;

	private boolean launchViewer = false;

	/**
	 * Main method which does the conversion.
	 * @throws Exception in case and error occurs
	 */
	public void convert() throws Exception {
		readOptions();
		logInfo("Converting to Oracle Help For Java(OHJ)..");
		
		// generate the multiple html files
		convertToMHTML(outputFile);
			
		// copy icons for the java help viewer to this folder
		logInfo("Copying icons to icons dir..");
		copyFiles(AURIGADOC_HOME + "/icons", outputFile + "/icons");

		// generate the hs file
		FileOutputStream fos = new FileOutputStream(outputFile + "/jhelp.hs");
		logInfo("Generating OHJ help set file ..");
		doTransform(new FileInputStream(inputFile), 
			fos, getXSLForKey(HELPSET_XSL, XML2OHJHS), OUTPUT_XML);
		fos.close();
			
		// generate the toc file
		fos = new FileOutputStream(outputFile + "/jhelptoc.xml");
		logInfo("Generating OHJ toc file..");
		doTransform(new FileInputStream(inputFile), 
			fos, getXSLForKey(TOC_XSL, XML2OHJTOC), OUTPUT_XML);
		fos.close();

		// generate the map file
		fos = new FileOutputStream(outputFile + "/Map.jhm");
		logInfo("Generating OHJ map file..");
		doTransform(new FileInputStream(inputFile), 
			fos, getXSLForKey(MAP_XSL, XML2OHJMAP), OUTPUT_XML);
		fos.close();

		if (indexFiles) {
			logInfo("Indexing files for use in OHJ ..");
			doIndexing(outputFile);
		}

		if (launchViewer) {
			logInfo("Launching files in a OHJ viewer..");
			launchHelpViewer(outputFile);
		}
	}


	private void doIndexing(String outDir) throws Exception {
		// index files
		if (!outDir.endsWith(File.separator)) {
			outDir += File.separator;
		}

		// create the config file.
		try {
			String indexFilePath = outDir + "/jhelp.idx"; 
			String[] indexerOptions = new String[2];
			indexerOptions[0] = outDir;
			indexerOptions[1] = indexFilePath;
			//oracle.help.tools.index.Indexer.main(indexerOptions);
			invokeMethod("oracle.help.tools.index.Indexer", "main", 
					indexerOptions);
		} catch (ClassNotFoundException e) {
			logWarning("Indexer not found: oracle.help.tools.index.Indexer");
		} catch (Exception e) {
			logError("Error indexing files: ", e);
		}
	}

	private void launchHelpViewer(String outDir) {
		try {
			String[] options = new String[2];
			options[0] = "-h";
			options[1] = outDir + "/jhelp.hs";
			//oracle.help.viewer.ViewerApplication.main(options);
			invokeMethod("oracle.help.viewer.ViewerApplication", "main", 
				options);
		} catch (ClassNotFoundException e) {
			logWarning("Viewer not found: oracle.help.viewer.ViewerApplication");
		} catch (Exception e) {
			logError("Error launching viewer: ", e);
		}
	}

	private void readOptions() {
		String indexFilesFlag = getParameter(INDEX_FILES);
		if (indexFilesFlag != null && indexFilesFlag.equals("y")) {
			indexFiles = true;
		}

		String launchViewerFlag = getParameter(LAUNCH_VIEWER);
		if (launchViewerFlag != null && launchViewerFlag.equals("y")) {
			launchViewer = true;
		}
	}

	private void invokeMethod(String className, String methodName, 
			String[] params) throws Exception {
		Class c = Class.forName(className);
		Class[] parameterTypes = new Class[] {String[].class};
		Method method;
		Object[] arguments = new Object[] {params};
		method = c.getMethod(methodName, parameterTypes);
		method.invoke(c, arguments);
	}
}
